const Me = imports.misc.extensionUtils.getCurrentExtension();
const { Gio, GLib, GObject } = imports.gi;
const { popupMenu, main, messageTray } = imports.ui;
const { QuickMenuToggle, SystemIndicator } = imports.ui.quickSettings;
const Log = Me.imports.modules.log;
const Resources = Me.imports.modules.resources;
var Title = "supergfxctl";
const gfxLabels = [
    "hybrid",
    "integrated",
    "vfio",
    "egpu",
    "asusmuxdiscreet",
    "none"
];
const gfxLabelsMenu = [
    "Hybrid",
    "Integrated",
    "VFIO",
    "eGPU",
    "MUX / dGPU",
    "None"
];
const powerLabel = [
    "active",
    "suspended",
    "off",
    "disabled",
    "active",
    "unknown",
];
const powerLabelFilename = [
    "active",
    "suspended",
    "off",
    "off",
    "active",
    "active",
];
const userAction = [
    "logout",
    "integrated",
    "asusgpumuxdisable",
    "none",
];
function spawnCommandLine(command) {
    try {
        GLib.spawn_command_line_async(command);
    }
    catch (e) {
        Log.error(`Spawning command failed: ${command}`, e);
    }
}
function notify(details, icon, action = "", urgency = 2) {
    let gIcon = Resources.getIcon(icon);
    let params = { gicon: gIcon };
    let source = new messageTray.Source("Super Graphics Control", icon, params);
    let notification = new messageTray.Notification(source, "Super Graphics Control", details, params);
    main.messageTray.add(source);
    notification.setTransient(true);
    if (action == "logout") {
        notification.setUrgency(3);
        notification.addAction("Log Out Now!", () => {
            spawnCommandLine("gnome-session-quit");
        });
    }
    else {
        notification.setUrgency(urgency);
    }
    source.showNotification(notification);
}
var gpuModeIndicator43 = GObject.registerClass(class gpuModeIndicator43 extends SystemIndicator {
    _init() {
        super._init();
    }
});
var gpuModeIndicator = GObject.registerClass(class gpuModeIndicator extends QuickMenuToggle {
    constructor() {
        super(...arguments);
        this.connected = false;
        this.pollerDelayTicks = 0;
        this.timeoutPollerGpuPower = null;
        this._supported = [];
        this._vendor = "";
        this._lastState = 6;
        this._powerState = 3;
        this._version = ["0", "0", "0"];
        this._versionTested = Me.metadata.supergfxctl.toString().split('.');
    }
    _init() {
        super._init();
        this._gpuModeItems = new Map();
        this._updateMenu = true;
        this._createProxy();
    }
    _createProxy() {
        let xml = Resources.File.DBus("org-supergfxctl-gfx-5");
        this._proxy = new Gio.DBusProxy.makeProxyWrapper(xml)(Gio.DBus.system, "org.supergfxctl.Daemon", "/org/supergfxctl/Gfx", async (proxy, e) => {
            if (e) {
                Log.error("Graphics Mode DBus initialization using supergfxctl failed!", e);
            }
            else {
                try {
                    this.connected = await this.getSupported();
                    if (!this.connected) {
                        throw "could not get supported GPU modes";
                    }
                    supergfxctlGexExtensionInstance.populate();
                    this.getVersion();
                    this.getSupported();
                    this.getVendor();
                    this.getGfxMode();
                    this.getGpuPower();
                    this.connectNotifySignal();
                    this.connectPowerSignal();
                    this._gpuModeSection = new popupMenu.PopupMenuSection();
                    this.menu.addMenuItem(this._gpuModeSection);
                    this.label = `${gfxLabelsMenu[this._lastState]} (${powerLabel[this._powerState]})`;
                    this.gicon = Resources.getIcon(`gpu-${gfxLabels[this._lastState]}${powerLabelFilename[this._powerState] == "active"
                        ? "-active"
                        : ""}`);
                    this.menu.setHeader(Resources.getIcon(`gpu-${gfxLabels[this._lastState]}${powerLabelFilename[this._powerState] == "active"
                        ? "-active"
                        : ""}`), `${gfxLabelsMenu[this._lastState]} (${powerLabel[this._powerState]})`);
                    this._sync();
                    return true;
                }
                catch (e) {
                    this.notifyError();
                    Log.error("Graphics Mode DBus initialization using supergfxctl failed after making the connection!", e);
                    return false;
                }
            }
        });
        return false;
    }
    notifyError() {
        let details = `Could not establish connection to supergfxctl. Is it installed correctly and can be executed as your user without root? Be sure to be in the wheel or user group, too. For further information have a visit at asus-linux.org or our helpful community at discord.`;
        let icon = 'gpu-integrated-active';
        notify(details, icon, '', 3);
    }
    _sync() {
        if (this._updateMenu) {
            this._gpuModeSection.removeAll();
            this._gpuModeItems.clear();
            if (gfxLabels[this._lastState] !== 'asusmuxdiscreet') {
                for (const key in this._supported) {
                    if (Object.prototype.hasOwnProperty.call(this._supported, key)) {
                        const element = this._supported[key];
                        const item = new popupMenu.PopupImageMenuItem(gfxLabelsMenu[element], Resources.getIcon(`gpu-${gfxLabels[element]}`));
                        item.connect("activate", () => {
                            this.setGfxMode(element);
                        });
                        this._gpuModeItems.set(element, item);
                        this._gpuModeSection.addMenuItem(item);
                    }
                }
                this._gpuModeSection.addMenuItem(new popupMenu.PopupSeparatorMenuItem());
            }
            for (const [profile, item] of this._gpuModeItems) {
                item.setOrnament(profile === this._lastState
                    ? popupMenu.Ornament.DOT
                    : popupMenu.Ornament.NONE);
            }
            if (this._version !== this._versionTested) {
                let versionText = '';
                if (this._version[0] !== this._versionTested[0]) {
                    versionText = `supergfxctl ${this._versionTested.join('.')}+ is required (${this._version.join('.')} installed)`;
                }
                else if (parseInt(this._version[1]) < parseInt(this._versionTested[1]) ||
                    parseInt(this._version[2]) < parseInt(this._versionTested[2])) {
                    versionText = `newer point release of supergfxctl ${this._versionTested.join('.')}+ is required (${this._version.join('.')} installed)`;
                }
                if (versionText.length > 0) {
                    const item = new popupMenu.PopupImageMenuItem(versionText, 'software-update-urgent-symbolic');
                    item.connect("activate", () => {
                        spawnCommandLine('xdg-open https://gitlab.com/asus-linux/supergfxctl/');
                    });
                    this._gpuModeItems.set(0, item);
                    this._gpuModeSection.addMenuItem(item);
                    this._gpuModeSection.addMenuItem(new popupMenu.PopupSeparatorMenuItem());
                }
            }
            let powerItem = new popupMenu.PopupImageMenuItem(`dedicated GPU is ${powerLabel[this._powerState]}`, Resources.getIcon(`dgpu-${powerLabelFilename[this._powerState]}`));
            powerItem.sensitive = false;
            powerItem.active = false;
            this._gpuModeSection.addMenuItem(powerItem);
            this.label = `${gfxLabelsMenu[this._lastState]} (${powerLabel[this._powerState]})`;
            this.gicon = Resources.getIcon(`gpu-${gfxLabels[this._lastState]}${powerLabelFilename[this._powerState] ==
                "active"
                ? "-active"
                : ""}`);
            this.setIndicator();
            this._updateMenu = false;
        }
        this.menu.setHeader(Resources.getIcon(`gpu-${gfxLabels[this._lastState]}${powerLabelFilename[this._powerState] == "active"
            ? "-active"
            : ""}`), `${gfxLabelsMenu[this._lastState]} (${powerLabel[this._powerState]})`);
    }
    setIndicator() {
        supergfxctlGexExtensionInstance.gpuModeIndicatorInstance.remove_actor(supergfxctlGexExtensionInstance.gpuModeIndicatorInstance._indicator);
        supergfxctlGexExtensionInstance.gpuModeIndicatorInstance._indicator.gicon = Resources.getIcon(`gpu-${gfxLabels[supergfxctlGexExtensionInstance.quickToggles._lastState]}${powerLabelFilename[supergfxctlGexExtensionInstance.quickToggles._powerState] == "active"
            ? "-active"
            : ""}`);
        supergfxctlGexExtensionInstance.gpuModeIndicatorInstance._indicator.style_class = "supergfxctl-gex panel-icon system-status-icon";
        supergfxctlGexExtensionInstance.gpuModeIndicatorInstance.add_actor(supergfxctlGexExtensionInstance.gpuModeIndicatorInstance._indicator);
    }
    connectNotifySignal() {
        try {
            this._proxy.connectSignal("NotifyAction", (proxy = null, name, value) => {
                let newMode = parseInt(this._proxy.ModeSync());
                let details = `Graphics Mode has changed.`;
                let icon = 'gpu-integrated-active';
                if (userAction[value] === "integrated") {
                    details = `You must switch to Integrated mode before switching to VFIO.`;
                }
                else if (userAction[value] !== "none") {
                    details = `Graphics Mode changed. Please save your work and ${userAction[value]} to apply the changes.`;
                    icon = 'reboot';
                    if (newMode !== this._lastState) {
                        this._lastState = newMode;
                        this._updateMenu = true;
                    }
                }
                else {
                    if (newMode !== this._lastState) {
                        details = `Graphics Mode changed to ${gfxLabels[newMode]}`;
                        this._lastState = newMode;
                        this._updateMenu = true;
                    }
                }
                this._sync();
                notify(details, icon, userAction[value]);
            });
        }
        catch (error) {
            Log.error(`Error connecting Signal, no live updates of GPU modes!`, error);
        }
    }
    connectPowerSignal() {
        try {
            this._proxy.connectSignal("NotifyGfxStatus", (proxy = null, name, value) => {
                if (value[0] !== this._powerState)
                    this.updatePanelPower(value[0]);
            });
        }
        catch (error) {
            Log.error(`Error connecting Signal, no live updates of GPU modes!`, error);
        }
    }
    getSupported() {
        try {
            let supported = this._proxy.SupportedSync();
            for (const [_key, _value] of Object.entries(supported)) {
                if (typeof _value == "object") {
                    for (const [_keyInner, _valueInner] of Object.entries(_value)) {
                        this._supported[parseInt(_keyInner)] = parseInt(_valueInner);
                    }
                }
            }
            return true;
        }
        catch (e) {
            Log.error("Graphics Mode DBus: get current mode failed!", e);
        }
        return false;
    }
    getVendor() {
        if (this.connected) {
            try {
                this._vendor = this._proxy.VendorSync();
                return this._vendor;
            }
            catch (e) {
                Log.error("Graphics Mode DBus: get current vendor failed!", e);
            }
        }
        return "";
    }
    getVersion() {
        if (this.connected) {
            try {
                const version = this._proxy.VersionSync();
                this._version = version.toString().split('.');
                return this._version;
            }
            catch (e) {
                Log.error("Graphics Mode DBus: get current version failed!", e);
            }
        }
        return ["0", "0", "0"];
    }
    getGfxMode() {
        if (this.connected) {
            try {
                this._lastState = parseInt(this._proxy.ModeSync());
                return this._lastState;
            }
            catch (e) {
                Log.error("Graphics Mode DBus: get current mode failed!", e);
            }
        }
        return 6;
    }
    setGfxMode(mode) {
        if (this.connected) {
            try {
                this._proxy.SetModeSync(mode);
                return true;
            }
            catch (e) {
                Log.error("Graphics Mode DBus switching failed!", e);
                supergfxctlGexExtensionInstance.notify(supergfxctlGexExtensionInstance.title, e.toString(), "gif/fire.gif");
            }
        }
        return false;
    }
    getGpuPower() {
        if (this.connected) {
            try {
                this._powerState = parseInt(this._proxy.PowerSync().toString().trim());
                return true;
            }
            catch (e) {
                Log.error("Graphics Mode DBus getting power mode failed!", e);
            }
        }
        this._powerState = 3;
        return false;
    }
    updatePanelPower(gpuPowerLocal = this._powerState) {
        if (gpuPowerLocal !== this._powerState) {
            this._powerState = gpuPowerLocal;
            if (gpuPowerLocal == 0 && this._lastState == 1) {
                try {
                    let mode = parseInt(this._proxy.ModeSync());
                    if (gfxLabels[mode] == "integrated")
                        supergfxctlGexExtensionInstance.notify(supergfxctlGexExtensionInstance.title, `Your dedicated GPU turned on while you are on the integrated mode. This should not happen. It could be that another application rescanned your PCI bus. Rebooting is advised.`, "gif/fire.gif", "reboot");
                    else if (this._lastState !== mode)
                        this._lastState = mode;
                }
                catch (e) {
                    Log.error("Graphics Mode DBus getting mode failed!", e);
                }
            }
            this._updateMenu = true;
            this._sync();
        }
    }
    stop() {
        this.connected = false;
        GLib.Source.remove(this.timeoutPollerGpuPower);
        this.timeoutPollerGpuPower = null;
        this._proxy = null;
    }
});
//# sourceMappingURL=panel43.js.map